<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Journal Plugin for Geeklog - The Ultimate Weblog                          |
// +---------------------------------------------------------------------------+
// | install.php                                                               |
// |                                                                           |
// | This file installs and removes the data structures for the Journal   |
// | plugin for Geeklog.                                                       |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2002 by the following authors:                              |
// |                                                                           |
// | Authors: Tony Bibbs       - tony@tonybibbs.com                            |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
//
// $Id: install.php,v 1.1.1.1 2003/06/10 20:33:24 dirk Exp $

require_once('../../../lib-common.php');
require_once($_CONF['path'] . 'plugins/journal/language/english.php');
require_once($_CONF['path'] . 'plugins/journal/journal.cfg');

// Only let Root users access this page
if (!SEC_inGroup('Root')) {
    // Someone is trying to illegally access this page
    COM_errorLog("Someone has tried to illegally access the Journal install/uninstall page.  User id: {$_USER['uid']}, Username: {$_USER['username']}, IP: $REMOTE_ADDR",1);
    $display = COM_siteHeader();
    $display .= COM_startBlock($LANG_JOURNAL['access_denied']);
    $display .= $LANG_JOURNAL['access_denied_msg'];
    $display .= COM_endBlock();
    $display .= COM_siteFooter();
    echo $display;
    exit;
}
 
/**
* Puts the datastructures for this plugin into the Geeklog database
*
*/
function plugin_install_journal()
{
    global $_TABLES, $_CONF, $_SP_CONF;

    COM_errorLog('Attempting to install the Journal Plugin',1);

    // Installs the Journal plugin
    $createsql = "CREATE TABLE {$_TABLES['journal']} ("
        . "jrn_id mediumint(8) DEFAULT '0' NOT NULL auto_increment,"
        . "jrn_uid mediumint(8) DEFAULT '1' NOT NULL,"
        . "jrn_name varchar(128) DEFAULT '' NOT NULL,"
        . "jrn_jt_id mediumint(8) DEFAULT '' NOT NULL,"
        . "jrn_public tinyint(3) unsigned DEFAULT '0' NOT NULL,"
        . "jrn_hits mediumint(8) unsigned DEFAULT '0' NOT NULL,"
        . "jrn_date datetime,"
        . "jrn_iscurrent tinyint(3) unsigned DEFAULT '0' NOT NULL,"
        . "PRIMARY KEY (jrn_id)"
        . ")";
    
    COM_errorLog("Attempting to create table journal for Journal plugin", 1);
    DB_query($createsql,1);

    if (DB_error()) {
        return false;
    }
    COM_errorLog('...success',1);
    $steps['createtablejournal'] = 1;
    
    // Installs the Journal plugin
    $createsql = "CREATE TABLE {$_TABLES['journal_type']} ("
        . "jt_id mediumint(8) DEFAULT '0' NOT NULL auto_increment,"
        . "jt_name varchar(64) DEFAULT '' NOT NULL,"
        . "PRIMARY KEY (jt_id)"
        . ")";
    
    COM_errorLog("Attempting to create table journal_type for Journal plugin", 1);
    DB_query($createsql,1);
    DB_query("INSERT INTO {$_TABLES['journal_type']} (jt_name) VALUES ('Personal Journal')");
    DB_query("INSERT INTO {$_TABLES['journal_type']} (jt_name) VALUES ('Work Journal')");

    if (DB_error()) {
        return false;
    }
    COM_errorLog('...success',1);
    $steps['createtablejournaltype'] = 1;
    
    // Installs the Journal plugin
    $createsql = "CREATE TABLE {$_TABLES['journal_entry']} ("
        . "je_id varchar(20) DEFAULT '' NOT NULL,"
        . "je_jrn_id mediumint(8) DEFAULT '0' NOT NULL,"
        . "je_summary varchar(128) DEFAULT '' NOT NULL,"
        . "je_text text,"
        . "je_public tinyint(3) unsigned DEFAULT '0' NOT NULL,"
        . "je_date datetime,"
        . "je_hits mediumint(8) unsigned DEFAULT '0' NOT NULL,"
        . "je_mood varchar(20),"
        . "PRIMARY KEY (je_id)"
        . ")";
    
    COM_errorLog("Attempting to create table journal_entry for Journal plugin", 1);
    DB_query($createsql,1);

    if (DB_error()) {
        return false;
    }
    COM_errorLog('...success',1);
    $steps['createtablejournalentry'] = 1;

    // Create the Journal admin security group
    COM_errorLog("Attempting to create Journal admin group", 1);
    DB_query("INSERT INTO {$_TABLES['groups']} (grp_name, grp_descr) "
        . "VALUES ('Journal Admin', 'Users in this group can administer the Journal plugin')",1);
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    COM_errorLog('...success',1);
    $steps['insertgroup'] = 1;

    // Save the grp id for later uninstall
    COM_errorLog('About to save group_id to vars table for use during uninstall',1);
    DB_query("INSERT INTO {$_TABLES['vars']} VALUES ('jrn_group_id', LAST_INSERT_ID())",1);
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    COM_errorLog('...success',1);
    $steps['savedgroupid'] = 1;

    $group_id = DB_getItem($_TABLES['vars'], 'value', "name = 'jrn_group_id'");

    // Add Journal features
    COM_errorLog('Attempting to add journal.edit feature',1);
    DB_query("INSERT INTO {$_TABLES['features']} (ft_name, ft_descr) "
        . "VALUES ('journal.edit','Access to Journal editor')",1);
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    $edit_id = DB_insertId();
    COM_errorLog('...success',1);
    $steps['insertedfeatureedit'] = 1;

    COM_errorLog('Attempting to add journal.delete feature',1);
    DB_query("INSERT INTO {$_TABLES['features']} (ft_name, ft_descr) "
        . "VALUES ('journal.delete','Ability to delete Journal')",1);
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    $delete_id = DB_insertId();
    COM_errorLog('...success',1);
    $steps['insertedfeaturedelete'] = 1;

    // Now add the features to the group
    COM_errorLog('Attempting to give Journal Admin group access to journal.edit feature',1);
    DB_query("INSERT INTO {$_TABLES['access']} (acc_ft_id, acc_grp_id) VALUES ($edit_id, $group_id)");
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    COM_errorLog('...success',1);
    $steps['addededittogroup'] = 1;

    COM_errorLog('Attempting to give Journal Admin group access to journal.delete feature',1);
    DB_query("INSERT INTO {$_TABLES['access']} (acc_ft_id, acc_grp_id) VALUES ($delete_id, $group_id)",1);
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    COM_errorLog('...success',1);
    $steps['addeddeletetogroup'] = 1;

    // OK, now give Root users access to this plugin now! NOTE: Root group should always be 1
    COM_errorLog('Attempting to give all users in Root group access Journal admin group',1);
    DB_query("INSERT INTO {$_TABLES['group_assignments']} VALUES ($group_id, NULL, 1)");
    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    COM_errorLog('...success',1);
    $steps['addedrootuserstogroup'] = 1;

    // Register the plugin with Geeklog
    if (DB_count($_TABLES['plugins'],'pi_name','journal') > 0) {
        COM_errorLog('Attempting to remove journal rentry prior to adding an updated entry',1);
        DB_query("DELETE FROM {$_TABLES['plugins']} WHERE pi_name = 'journal'");
        if (DB_error()) {
            plugin_uninstall_journal($steps);
            return false;
            exit;
        }
        COM_errorLog('...success',1);
    } else {
        // Only install data on a fresh installation
        // This plugin has no install data
    }

    COM_errorLog('Registering Journal plugin with Geeklog', 1);
    DB_delete($_TABLES['plugins'],'pi_name','journal');
    DB_query("INSERT INTO {$_TABLES['plugins']} (pi_name, pi_version, pi_gl_version, pi_homepage, pi_enabled) "
        . "VALUES ('journal', '{$_JRN_VERSION}', '1.3.6', 'http://www.tonybibbs.com', 1)");
    DB_query("INSERT INTO {$_TABLES['vars']} VALUES ('journal','1')");

    if (DB_error()) {
        plugin_uninstall_journal($steps);
        return false;
        exit;
    }
    COM_errorLog('...success',1);
    COM_errorLog('Succesfully installed the Journal Plugin!',1);

    return true;
}

/**
* Removes the datastructures for this plugin from the Geeklog database
*
* This may get called by the install routine to undue anything done to this
* point.  To do that, $steps will have a list of steps to undo
*
* @steps   Array    Holds all the steps that have been completed by the install
*
*/  
function plugin_uninstall_journal($steps = '')
{
    global $_TABLES;

    // Uninstalls the Journal plugin

    if (empty($steps) OR $steps['createtablejournal'] == 1) {
        // Remove the journal table 
        COM_errorLog('Dropping journal table',1);
        DB_query("DROP TABLE {$_TABLES['journal']}");
        COM_errorLog('...success',1);
    }
    
    if (empty($steps) OR $steps['createtablejournalentry'] == 1) {
        // Remove the journal table 
        COM_errorLog('Dropping journal_entry table',1);
        DB_query("DROP TABLE {$_TABLES['journal_entry']}");
        COM_errorLog('...success',1);
    }
    
    if (empty($steps) OR $steps['createtablejournaltype'] == 1) {
        // Remove the journal table 
        COM_errorLog('Dropping journal_type table',1);
        DB_query("DROP TABLE {$_TABLES['journal_type']}");
        COM_errorLog('...success',1);
    }

    // Remove security for this plugin

    // Remove the Journal admin group
    $grp_id = DB_getItem($_TABLES['vars'], 'value', "name = 'jrn_group_id'");

    if (empty($steps) OR $steps['insertgroup'] == 1) {
        COM_errorLog('Attempting to remove the Journal Admin Group', 1);
        DB_query("DELETE FROM {$_TABLES['groups']} WHERE grp_id = $grp_id");
        COM_errorLog('...success',1);
    }

    // Remove related features
    $edit_id = DB_getItem($_TABLES['features'], 'ft_id', "ft_name = 'journal.edit'");
    $delete_id = DB_getItem($_TABLES['features'], 'ft_id', "ft_name = 'journal.delete'");

    if (empty($steps) OR $steps['addededittogroup'] == 1) {
        // Remove access to those features 
        COM_errorLog('Attempting to remove rights to journal.edit from all groups',1);
        DB_query("DELETE FROM {$_TABLES['access']} WHERE acc_ft_id = $edit_id");
        COM_errorLog('...success',1);
    }

    if (empty($steps) OR $steps['addeddeletetogroup'] == 1) {
        // Remove access to those features 
        COM_errorLog('Attempting to remove rights to journal.delete from all groups',1);
        DB_query("DELETE FROM {$_TABLES['access']} WHERE acc_ft_id = $delete_id");
        COM_errorLog('...success',1);
    }

    if (empty($steps) OR $steps['addedrootuserstogroup'] == 1) {
        // Remove root users from the group
        COM_errorLog('Attempting to remove root users from admin of Journal');
        DB_query("DELETE FROM {$_TABLES['group_assignments']} WHERE ug_main_grp_id = $grp_id");
        COM_errorLog('...success',1);
    }

    if (empty($steps) OR $steps['insertedfeatureedit'] == 1) {
        COM_errorLog('Attempting to remove the journal.edit feature',1);
        DB_query("DELETE FROM {$_TABLES['features']} WHERE ft_id = $edit_id");
        COM_errorLog('...success',1);
    }

    if (empty($steps) OR $steps['insertedfeaturedelete'] == 1) {
        COM_errorLog('Attempting to remove the journal.delete feature',1);
        DB_query("DELETE FROM {$_TABLES['features']} WHERE ft_id = $delete_id");
        COM_errorLog('...success',1);
    }

    if (empty($steps) OR $steps['savedgroupid']) {
        COM_errorLog('Attempting to remove the group id from the vars table',1);
        DB_query("DELETE FROM {$_TABLES['vars']} WHERE name = 'jrn_group_id'");
        COM_errorLog('success',1);
    }

    // Unregister the plugin with Geeklog
    if (empty($steps)) {
        COM_errorLog('Attempting to unregister the plugin from Geeklog',1);
        DB_query("DELETE FROM {$_TABLES['plugins']} WHERE pi_name = 'journal'");
        DB_query("DELETE FROM {$_TABLES['vars']} WHERE name = 'journal'");
        COM_errorLog('...success',1);

        COM_errorLog('leaving plugin_uninstall_journal',1);
        return true;
    } else {
        return false;
    }
}

/* 
MAIN:  OK, I wanted to keep this simple so this is how this works.  When ran for the first
time, this file will attempt to install the datastructures immediately.  When that is complete,
it will insert a record into the Geeklog vars table (name = 'journal', value = 1) which will
lock this file from being executed at by anyone.

If you want to remove this plugin you would set the value in that record to 0 and then this script
will uninstall the plugin.  All this is outlined in the INSTALL and UNINSTALL documents.
*/

$display = COM_siteHeader();

if (DB_count($_TABLES['vars'], 'name', 'journal') == 0) {
    // Record in vars table doesn' exit, install this plugin
    if (plugin_install_journal()) {
        $display .= COM_startBlock($LANG_JOURNAL['installation_complete']);
        $display .= $LANG_JOURNAL['installation_complete_msg'];
        $display .= COM_endBlock();
    } else {
        // Error occured
        $display .= COM_startBlock($LANG_JOURNAL['installation_failed']);
        $display .= $LANG_JOURNAL['installation_failed_msg'];
        $display .= COM_endBlock();
    }   
} else {
    // This plugin is installed, see if it is locked or not
    if (DB_getItem($_TABLES['vars'],'value',"name = 'journal'") == 1) {
        // This is locked, do nothing
        $display .= COM_startBlock($LANG_JOURNAL['system_locked']);
        $display .= $LANG_JOURNAL['system_locked_msg'];
        $display .= COM_endBlock();
    } else {
        // Uninstall plugin
        if (plugin_uninstall_journal()) {
            // Uninstall worked
            $display .= COM_startBlock($LANG_JOURNAL['uninstall_complete']);
            $display .= $LANG_JOURNAL['uninstall_complete_msg'];
            $display .= COM_endBlock();
        } else {
            // Uninstall failed
            $display .= COM_startBlock($LANG_JOURNAL['uninstall_failed']);
            $display .= $LANG_JOURNAL['uninstall_failed_msg'];
            $display .= COM_endBlock();
        }
    }
}

$display .= COM_siteFooter();

echo $display;

?>
